#! /usr/bin/env python

import sys

from numpy import *
from numpy.fft import ifft
import numpy.random as random

## global configuration and variables

max_chnls = 16
Xn = []
yn = []

for i in range(max_chnls):
    Xn.append(array([]))
    yn.append(array([]))

## Csound functions
def new_seed(seed_in = -1):
#    print "New seed: ", int(seed_in)
    if seed_in == -1:
        random.seed()  #can put seed here, leaving blank uses system time
    else:
        random.seed(longlong(seed_in))  #can put seed here, leaving blank uses system time
    return float(seed_in);

def get_ir_length(channel = -1):
    index = int(channel) if channel != -1 else 1
    return float(len(Xn[index]))

def get_ir_point(channel, index):
    global yn
    return float(yn[int(channel - 1)][int(index)].real)

def new_ir_for_channel(N, channel = 1, max_jump=-1):
    global Xn, yn
    [Xfinal, y] = new_ir(N, max_jump)
    Xn[int(channel) - 1] = Xfinal
    yn[int(channel) - 1] = y

def new_ir(N, max_jump=-1):
#    '''   new_ir(N)
#    N - is the number of points for the IR,
#    max_jump -  is the maximum phase difference (in radians) between bins
#             if -1, the random numbers are used directly (no jumping).
#   '''
    if N < 16:
        print "Warning: N is too small."
    print "Generate new IR size=", N
    n = N//2 # before mirroring
    Am = ones((n))
    Ph = array([])
    limit = pi

    Ph = append(Ph, 0)
    old_phase = 0
    for i in range(1,n):
        if max_jump == -1:
            Ph = append(Ph, (random.random()* limit) - (limit/2.0))
        else:
        # make phase only move +- limit
            delta = (random.random() * max_jump * 2* pi) - (max_jump * pi)
            new_phase = old_phase + delta
            Ph = append(Ph, new_phase)
            old_phase = new_phase
    #pad DC to 0 and double last bin
    Am[0] = 0
    Xreal = multiply(Am, cos(Ph))
    Ximag = multiply(Am, sin(Ph))
    X = Xreal + (1j*Ximag)

    Xsym = conj(X[1:n])[::-1] # reverse the conjugate
    X = append(X, X[0])
    Xfinal = append(X,Xsym)
    y = ifft(Xfinal)
    return [Xfinal, y.real]

